#include "syncscheduler_sfy.hpp"
#include "OneDayWindow.hpp"
#include "MainWindow.hpp"
#include "syncscheduler_sfy.brh"
#include "SoftMenuID.hpp"
#include "ColorDef.hpp"

OneDayWindow::OneDayWindow() static_throws
{
    if (static_try()) {
        // \tgL[nh̓o^
        static_throw(RegisterHandler(
            SFXEventRange(SFEVT_RESPONDER_SOFTKEY, SFEVT_RESPONDER_SOFTKEY, AVK_SOFT1, AVK_SOFT4),
            XANDLER_INTERNAL(OnSoftKey)
        ));
        if (static_try()) {
            // \tgL[j[̐ݒ
            static_throw(SetSoftMenu());
            if (static_try()) {
                // _CAOwp[̎擾
                if ((_dialoger = syncscheduler_sfy::GetDialoger()) != null) {
                    _vcalCollection = syncscheduler_sfy::GetVCalCollection();
                }
                else {
                    static_throw(SFERR_FAILED);
                }
            }
        }
    }
    return;
}

// fXgN^
OneDayWindow::~OneDayWindow(Void)
{
}

// CX^X̍쐬
OneDayWindowSmp OneDayWindow::NewInstance(SFCErrorPtr exception)
{
    return static_pointer_cast<OneDayWindow>(Factory(::new OneDayWindow(), exception));
}

// X|_̍쐬
SFCError OneDayWindow::Make(SFXDateConstRef date)
{
    _cursor = date;
    SFCError error;

    if ((error = CreateScheduleMenu()) == SFERR_NO_ERROR) {
        if ((error = RefreshMenu()) == SFERR_NO_ERROR) {
            if (_menu->GetItemCount() == 0) {
                if ((error = CreateFrame(GetParentFrameTitle())) == SFERR_NO_ERROR) {
                    error = CreateDataInputWindow(null);
                }
            }
        }
    }
    return error;
}

// \tgL[j[̐ݒ
SFCError OneDayWindow::SetSoftMenu(Void)
{
    SFCError error(SFERR_NO_ERROR);

    if ((_softkey = syncscheduler_sfy::GetSoftKey()) != null) {
        if (!_softkey->ContainsMenuKey(SOFTMENU_ONEDAY)) {
            if ((error = _softkey->CreateMenu(SOFTMENU_ONEDAY)) == SFERR_NO_ERROR) {
                UInt32 key[] = {IDS_SOFTKEY_EDIT, IDS_SOFTKEY_RETURN, IDS_SOFTKEY_MENU};
                error = _softkey->SetTextKey(SOFTMENU_ONEDAY, key, lengthof(key));
            }
        }
        if (error == SFERR_NO_ERROR) {
            error = _softkey->Bind(this, SOFTMENU_ONEDAY);
        }
    }
    else {
        error = SFERR_FAILED;
    }
    return error;
}

// t[̍쐬Ɣzu
SFCError OneDayWindow::CreateFrame(SFXAnsiStringConstRef title)
{
    SFCError error;

    if ((_frame = SFZTitlePlainFrame::NewInstance(&error)) != null) {
        if ((error = _frame->SetParent(GetRoot())) == SFERR_NO_ERROR) {
            if ((error = _frame->SetText(title)) == SFERR_NO_ERROR) {
                _frame->SetRealBound(syncscheduler_sfy::GetContentRect());
                _frame->SetState(true, true, true, true);
                _frame->ToFront();
            }
        }
        if (error != SFERR_NO_ERROR) {
            _frame->Terminate();
        }
    }
    return error;
}

// \胁j[̍쐬
SFCError OneDayWindow::CreateScheduleMenu(Void)
{
    static SFXEventRange::AtomRecConst range[] = {
        {SFEVT_RESPONDER_RESULT, SFEVT_RESPONDER_RESULT, SFP16_BEGIN, SFP16_END}
    };
    SFYHandler::RuleRec rule[lengthof(range)];
    SFCError error;

    if ((_menu = SFZTextMenu::NewInstance(&error)) != null) {
        if ((error = _menu->SetParent(GetThis())) == SFERR_NO_ERROR) {
            rule[0].spp = XANDLER_FUNCTION(OnMenu);
            rule[0].reference = this;
            if ((error = _menu->RegisterHandler(atomic_cast(range), rule, lengthof(range))) == SFERR_NO_ERROR) {
                _menu->SetTitleHeight(0);
                _menu->SetSelBackColor(COLOR_MENU_SEL_BACK);
                _menu->SetRealBound(GetLocalBound());
                _menu->SetState(true, true, true, true);
            }
        }
    }
    return error;
}

// \胁j[̍XV
SFCError OneDayWindow::RefreshMenu(Void)
{
    SFXAnsiString title;
    SIntN i;
    SFCError error(SFERR_NO_ERROR);

    _menu->Clear();
    if (_vcalCollection->IsDataExist(_cursor.GetYear(), _cursor.GetMonth(), _cursor.GetDay())) {
        _todayIndex = _vcalCollection->GetIndex(_cursor.GetYear(), _cursor.GetMonth(), _cursor.GetDay());
        _todaySize = _vcalCollection->GetNumberOfVCal(_todayIndex, _cursor.GetYear(), _cursor.GetMonth(), _cursor.GetDay());
        for (i = 0; i < _todaySize; i++) {
            VCalendarItemPtr vcal;
            vcal = _vcalCollection->Get(_todayIndex + i);
            title = SFXAnsiString::Format("%02d:%02d ", vcal->GetStartDate().GetHour(), vcal->GetStartDate().GetMinute());
            title += vcal->GetSummary();
            if ((error = _menu->AppendItem(title)) == SFERR_NO_ERROR) {
                _menu->SetItemBackColor(i, (i % 2) ? COLOR_MENU_ITEM_ODD_BACK : COLOR_MENU_ITEM_EVEN_BACK);
            }
            else {
                break;
            }
        }
    }
    return error;
}

// f[^̓EBhE̍쐬Ɣzu
SFCError OneDayWindow::CreateDataInputWindow(VCalendarItemPtr vcal)
{
    DataInputWindowSmp datainput;
    SFCError error;

    if ((datainput = DataInputWindow::NewInstance(&error)) != null) {
        if ((error = datainput->SetParent(_frame)) == SFERR_NO_ERROR) {
            error = datainput->RegisterHandler(
                SFXEventRange(SFEVT_RESPONDER_STATE, SFEVT_RESPONDER_STATE, SFP16_STATE_VALID, SFP16_STATE_VALID),
                XANDLER_INTERNAL(OnDataInputWindowValid)
            );
            if (error == SFERR_NO_ERROR) {
                datainput->SetRealBound(_frame->GetLocalBound().Deflate(_frame->GetSuitableMargin()));
                datainput->SetState(true, true, true, true);
                error = datainput->Make(_cursor, vcal);
            }
        }
    }
    return error;
}

// Tuj[̍쐬Ɣzu
SFCError OneDayWindow::CreateSubMenu(Void)
{
    static SFXEventRange::AtomRecConst range[] = {
        {SFEVT_RESPONDER_RESULT, SFEVT_RESPONDER_RESULT, SFP16_BEGIN, SFP16_END},
        {SFEVT_RESPONDER_SOFTKEY, SFEVT_RESPONDER_SOFTKEY, AVK_SOFT1, AVK_SOFT1}
    };
    SFYHandler::RuleRec rule[lengthof(range)];
    SFZTextMenuSmp submenu;
    SFXRectangle rc(syncscheduler_sfy::GetContentRect());
    SFCError error;

    if ((submenu= SFZTextMenu::NewInstance(&error)) != null) {
        if ((error = submenu->SetParent(GetRoot())) == SFERR_NO_ERROR) {
            rule[0].spp = XANDLER_FUNCTION(OnSubMenu);          // Tuj[̃nh
            rule[0].reference = this;
            rule[1].spp = XANDLER_FUNCTION(OnSubMenuSoftKey);   // Tuj[̃\tgL[nh
            rule[1].reference = this;
            if ((error = submenu->RegisterHandler(atomic_cast(range), rule, lengthof(range))) == SFERR_NO_ERROR) {
                if ((error = submenu->SetTitle(syncscheduler_sfy::LoadString(IDS_MENU))) == SFERR_NO_ERROR) {
                    submenu->SetDefaultItemBackColor(COLOR_MENU_ITEM_EVEN_BACK);
                    submenu->SetTitleForeColor(COLOR_MENU_TITLE_FORE);
                    submenu->SetTitleBackColor(COLOR_MENU_TITLE_BACK);
                    submenu->SetSelBackColor(COLOR_MENU_SEL_BACK);
                    if ((error = submenu->AppendItem(syncscheduler_sfy::LoadString(IDS_PLAN_NEW))) == SFERR_NO_ERROR) {
                        if ((error = submenu->AppendItem(syncscheduler_sfy::LoadString(IDS_PLAN_DELETE))) == SFERR_NO_ERROR) {
                            rc.SetTop(rc.GetBottom() - (submenu->GetTitleHeight() + submenu->GetItemHeight() * (SInt16)submenu->GetItemCount()));
                            rc.SetLeft(rc.GetRight() - rc.GetWidth() / 2);
                            submenu->SetRealBound(rc);
                            submenu->SetState(true, true, true, true);
                            submenu->ToFront();
                            error = _softkey->Bind(submenu.Get(), SOFTMENU_SUBMENU);
                        }
                    }
                }
            }
        }
    }
    return error;
}

// ẽt[^Cg擾
SFXWideStringConstRef OneDayWindow::GetParentFrameTitle(Void)
{
    SFZTitlePlainFrameSmp frame(static_pointer_cast<SFZTitlePlainFrame>(GetParent()));
    SFXWideStringConstPtr text;

    if (frame != null) {
        text = &frame->GetText();
    }
    else {
        text = &SFXWideString::EmptyInstance();
    }
    return *text;
}

// ẽt[ƃEBhEI
Void OneDayWindow::TerminateParentFrame(Void)
{
    SFYResponderSmp parent(GetParent());

    if (parent != null) {
        parent->Terminate();
    }
    return;
}

// j[nh
XANDLER_IMPLEMENT_VOIDRESULT(OneDayWindow, OnMenu, invoker, reason, result)
{
    VCalendarItemPtr vcal(null);
    SFCError error;

    unused(invoker); unused(result);
    switch (reason) {
        case SFP16_RESULT_OK:
            if ((error = CreateFrame(GetParentFrameTitle())) == SFERR_NO_ERROR) {
                vcal = _vcalCollection->Get(_todayIndex + _menu->GetSelect());
                if ((error = CreateDataInputWindow(vcal)) != SFERR_NO_ERROR) {
                    _frame->Terminate();
                }
            }
            break;
        case SFP16_RESULT_CANCEL:
        case SFP16_RESULT_ESCAPE:
            TerminateParentFrame();
            break;
    }
    return;
}

// \tgL[nh
XANDLER_IMPLEMENT_VOIDEVENT(OneDayWindow, OnSoftKey, invoker, event)
{
    SFCError error;

    unused(invoker);
    switch (event.GetP16()) {
        case AVK_SOFT1: // 폜
            TerminateParentFrame();
            break;
        case AVK_SOFT2: // Tuj[
            error = CreateSubMenu();
            break;
    }
    return;
}

// 폜nh
XANDLER_IMPLEMENT_VOIDRESULT(OneDayWindow, OnDelete, invoker, reason, result)
{
    VCalendarItemPtr vcal = _vcalCollection->Get(_todayIndex + _menu->GetSelect());

    unused(invoker); unused(result);
    if (vcal != null) {
        switch (reason) {
            case SFP16_RESULT_OK:
                _vcalCollection->Remove(vcal);
                _menu->SetSelect(_menu->GetSelect());
                if (RefreshMenu() == SFERR_NO_ERROR) {
                    if (_menu->GetItemCount() == 0) {
                        TerminateParentFrame();
                    }
                }
                break;
        }
    }
    _dialoger->Terminate();
    return;
}

// f[^̓EBhELԃnh
XANDLER_IMPLEMENT_VOIDSTATE(OneDayWindow, OnDataInputWindowValid, invoker, reason, state)
{
    unused(invoker); unused(reason);
    if (!state) {
        if (RefreshMenu() == SFERR_NO_ERROR) {
            if (_menu->GetItemCount() == 0) {
                TerminateParentFrame();
            }
        }
    }
    return;
}

// Tuj[̃nh
XANDLER_IMPLEMENT_VOIDRESULT(OneDayWindow, OnSubMenu, invoker, reason, result)
{
    unused(invoker);
    SFCError error;

    if (reason == SFP16_RESULT_OK) {
        switch (result) {
        case ITEM_NEW:      // VK
            if ((error = CreateFrame(GetParentFrameTitle())) == SFERR_NO_ERROR) {
                error = CreateDataInputWindow(null);
            }
            break;
        case ITEM_DELETE:   // 폜
            if (_menu->GetItemCount() > 0) {
                _dialoger->SetTitle(Dialoger::CONFIRM);
                _dialoger->SetMessageText(syncscheduler_sfy::LoadString(IDS_DELETE_CONFIRM));
                _dialoger->SetIconImage(SFBShell::GetInstance()->LoadResImage(SYNCSCHEDULER_SFY_RES_FILE, IDI_ACCEPT));
                _dialoger->SetButtonState(Dialoger::OK | Dialoger::CANCEL);
                _dialoger->RegisterHandler(XANDLER_INTERNAL(OnDelete));
                error = _dialoger->Execute();
            }
            break;
        }
    }
    invoker->Terminate();
    return;
}

// Tuj[̃\tgL[nh
XANDLER_IMPLEMENT_VOIDRESULT(OneDayWindow, OnSubMenuSoftKey, invoker, reason, result)
{
    unused(result);
    switch (reason) {
        case AVK_SOFT1:
            invoker->Terminate();
            break;
    }
    return;
}
