#include "VCalendar.hpp"

#define DATE_FORMAT         "YYYYMMDD%Thhmmss"  // ttH[}bg
#define DIR_NAME            "/calendar/"        // fBNg
#define DATA_FILE_NAME      "data.vcs"          // VCalendar t@C

// ftHg RXgN^
VCalendarItem::VCalendarItem(Void)
{
}

// RXgN^
VCalendarItem::VCalendarItem(SFXAnsiStringConstRef string)
{
    Import(string);
}

// J_[̏
Void VCalendarItem::Clear(Void)
{
    _startDate.Set(0);
    _endDate.Set(0);
    _lastModifiedDate.Set(0);
    _summary.Clear();
    _others.Clear();
}

// vCalendar`̕荞
SFCError VCalendarItem::Import(SFXAnsiStringConstRef string)
{
    SFXAnsiString field;    // "***:***" 
    SFXAnsiString value;    // "***:***" E
    SInt32 c1;
    SInt32 c2;
    SFCError error(SFERR_NO_ERROR);

    Clear();
    if ((c1 = string.FirstIndexOf("BEGIN:VEVENT", 0, false)) >= 0) {
        c2 = string.FirstIndexOf('\n', c1);
        while (true) {
            if ((c1 = string.FirstIndexOf(':', c2)) >= 0) {
                field = string.Substring(c2 + 1, c1).Trim();
                if ((c2 = string.FirstIndexOf('\n', c1)) < 0) {     // sȂ
                    c2 = string.GetLength();                        // Ō
                }
                value = string.Substring(c1 + 1, c2).Trim();
                if (field.Equals("CLASS", false)) {
                    if (value.Equals("PUBLIC", false)) {
                        ;                                           // 
                    }
                }
                else if (field.Equals("DTSTART", false)) {
                    _startDate.Parse(DATE_FORMAT, value);           // Jn
                }
                else if (field.Equals("DTEND", false)) {
                    _endDate.Parse(DATE_FORMAT, value);             // I
                }
                else if (field.Equals("LAST-MODIFIED", false)) {
                    _lastModifiedDate.Parse(DATE_FORMAT, value);    // XV
                }
                else if (field.Substring(0, 7).Equals("SUMMARY", false)) {
                    _summary = value;                               // v
                }
                else if (field.Equals("END", false)) {              // CxgI^O
                    if (value.Equals("VEVENT", false)) {            // I
                        break;
                    }
                }
                else {
                    _others << field << ":" << value << "\r\n";     // ̑
                }
            }
            else {                                                  // ":" Ȃ΁A
                break;                                              // I
            }
        }
    }
    return error;
}

// vCalendar`̃f[^oB
SFXAnsiString VCalendarItem::Export(Void) const
{
    SFXAnsiString result;

    result += "BEGIN:VEVENT\r\n";
    result += "CLASS:PUBLIC\r\n";
    result += SFXAnsiString::Format("DTSTART:%s\r\n", _startDate.Format(DATE_FORMAT).GetCString());
    result += SFXAnsiString::Format("DTEND:%s\r\n", _endDate.Format(DATE_FORMAT).GetCString());
    result += SFXAnsiString::Format("SUMMARY;ENCODING=QUOTED-PRINTABLE:%s\r\n", _summary.GetCString());
    result += SFXAnsiString::Format("LAST-MODIFIED:%s\r\n", _lastModifiedDate.Format(DATE_FORMAT).GetCString());
    result += _others;
    result += "END:VEVENT\r\n";
    return result;
}

// RXgN^
VCalendarCollection::VCalendarCollection(Void)
{
}

// fXgN^
VCalendarCollection::~VCalendarCollection(Void)
{
    Clear();
}

// ACeS
Void VCalendarCollection::Clear(Void)
{
    SFXList<VCalendarItemPtr>::Enumerator e = _list.GetFirstEnumerator();

    while (e.HasNext()) {
        ::delete e.GetNext();
    }
    _list.Clear();
    return;
}

// NXf[^ vCalendar `̃t@Cɕۑ
SFCError VCalendarCollection::Save(Void) const
{
    SFXFile file;
    SFXAnsiStringStreamWriter writer;
    SFXAnsiString content;
    SFXPath filePath(DIR_NAME DATA_FILE_NAME);
    SFXPath tempPath;
    SFCError error;

    if ((error = SFXFile::GetTemporaryPath(SFXPath(DIR_NAME), &tempPath)) == SFERR_NO_ERROR) {
        if ((error = file.OpenReadWrite(tempPath)) == SFERR_NO_ERROR) {
            SFXList<VCalendarItemPtr>::Enumerator e = _list.GetFirstEnumerator();
            SFXAnsiString content;
            content += "BEGIN:VCALENDAR\r\n";
            content += "VERSION:1.0\r\n";
            while (e.HasNext()) {
                VCalendarItemPtr vcal = e.GetNext();
                content += vcal->Export();
            }
            content += "END:VCALENDAR\r\n";
            if ((error = file.GetStreamWriter(content.GetLength(), &writer)) == SFERR_NO_ERROR) {
                writer << content;
                error = writer.Flush();
            }
            writer.Release();
            file.Close();
        }
        if (error == SFERR_NO_ERROR) {
            error = SFXFile::Rename(tempPath, filePath);
        }
        else {
            SFXFile::Remove(tempPath);
        }
    }
    return error;
}

// vCalendar `̃t@CNXɃf[^ǂݍ
SFCError VCalendarCollection::Load(Void)
{
    SFXFile file;
    SFXAnsiStringStreamReader reader;
    SFXAnsiString content;
    SFXPath filePath(DIR_NAME DATA_FILE_NAME);
    UInt32 size;
    SFCError error;

    Clear();
    if ((error = SFXFile::GetSize(filePath, &size)) == SFERR_NO_ERROR) {
        if ((error = file.OpenReadOnly(filePath)) == SFERR_NO_ERROR) {
            if ((error = file.GetStreamReader(size, &reader)) == SFERR_NO_ERROR) {
                error = reader.Fetch();
                reader >> content;
            }
            reader.Release();
            file.Close();
        }
    }
    if (error == SFERR_NO_ERROR) {
        SInt32 cs = 0;
        SInt32 ce = 0;
        while (true) {
            if ((cs = content.FirstIndexOf("BEGIN:VEVENT", ce, false)) >= 0) {
                if ((ce = content.FirstIndexOf("END:VEVENT", cs, false)) >= 0) {
                    ce = ce + 10;
                    error = InsertLast(::new VCalendarItem(content.Substring(cs, ce)));
                }
                else {
                    break;
                }
            }
            else {
                break;
            }
        }
    }
    return SFERR_NO_ERROR;
}

// VCalendarCollection ɃACe
SFCError VCalendarCollection::InsertLast(VCalendarItemPtr vcal)
{
    SFXList<VCalendarItemPtr>::Iterator it = _list.GetFirstIterator();
    SFXDateConst date(vcal->GetStartDate());
    Bool isInsert(false);
    SFCError error(SFERR_NO_ERROR);

    for (SIntN i = 0; it.HasNext(); i++) {
        VCalendarItemPtr temp = it.GetNext();
        if (date <= temp->GetStartDate()) { // t̏ɂȂ悤
            error = _list.Insert(i, vcal);  // }
            isInsert = true;
            break;
        }
    }
    if (!isInsert) {                        // }ȂA
        error = _list.InsertLast(vcal);     // Ōɒǉ
    }
    return error;
}

// ACe̍XV
SFCError VCalendarCollection::Update(VCalendarItemPtr vcal)
{
    _list.Remove(_list.FirstIndexOf(vcal));
    return InsertLast(vcal);
}

// ACeSɖ
Void VCalendarCollection::Remove(VCalendarItemPtr vcal)
{
    ::delete vcal;
    _list.Remove(_list.FirstIndexOf(vcal));
    return;
}

// index ACe擾
VCalendarItemPtr VCalendarCollection::Get(SInt32 index) const
{
    return _list.Get(index);
}

// year NAmonth Aday uȍ~v̍ŏ̃f[^ index Ŏ擾
SInt32 VCalendarCollection::GetIndex(SInt16 year, SInt16 month, SInt16 day) const
{
    SFXDate date1(year, month, day);
    SFXDate date2;
    SInt32 i;

    for (i = 0 ; i < _list.GetSize() ; i++) {
        VCalendarItemPtr vcal = _list.Get(i);
        date2 = vcal->GetStartDate();
        date2.SetTime(0, 0, 0);
        if (date1 <= date2) {
            break;
        }
    }
    return i;
}

// year NAmonth Aday ̃f[^̌擾
SInt32 VCalendarCollection::GetNumberOfVCal(SInt16 year, SInt16 month, SInt16 day) const
{
    return GetNumberOfVCal(GetIndex(year, month, day), year, month, day);
}

// index w肵 year NAmonth Aday ̃f[^̌擾
SInt32 VCalendarCollection::GetNumberOfVCal(SInt32 index, SInt16 year, SInt16 month, SInt16 day) const
{
    SFXList<VCalendarItemPtr>::Enumerator e = _list.GetEnumerator(index);
    SFXDate date1(year, month, day);
    SFXDate date2;
    SInt32 size;

    for (size = 0; e.HasNext(); ++size) {
        VCalendarItemPtr vcal = e.GetNext();
        date2 = vcal->GetStartDate();
        date2.SetTime(0, 0, 0);
        if (date1 != date2) {
            break;
        }
    }
    return size;
}

// year NAmonth Aday ̃f[^݂邩ǂ
Bool VCalendarCollection::IsDataExist(SInt16 year, SInt16 month, SInt16 day) const
{
    SFXList<VCalendarItemPtr>::Enumerator e = _list.GetEnumerator(GetIndex(year, month, day));
    Bool result(false);

    if (e.HasNext()) {
        SFXDate date = e.GetNext()->GetStartDate();
        if (date.GetYear() == year && date.GetMonth() == month && date.GetDay() == day) {
            result = true;
        }
    }
    return result;
}
